@@ -1,24 +1,22 @@
 #encoding: utf-8
-"""
-Copyright (C) 2012 Wesley Hansen
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-Contact via email: wes@ridersdiscount.com
-"""
+#Copyright (C) 2012 Wesley Hansen
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+#Contact via email: wes@ridersdiscount.com
 
 __author__ = "Wesley Hansen"
-__version__ = "0.1"
+__version__ = "0.1.3"
 __date__ = "04/20/2012 12:38:22 PM"
 
 
@@ -26,23 +24,26 @@ import gtk
 import sys
 import simplejson
 import os
+import pprint
+import copy
 
 
 class InvalidFilePathError(Exception):pass
 class InvalidRendererList(Exception):pass
+class ConfigValidationError(Exception):pass
 
 class ConfigTreeView( gtk.TreeView ):
-	"""
-	A `gtk.TreeView` wrapper that initializes its columns, renderers and properties through
-	the use of a configuration structure. The `gtk.TreeView` itself handles assigning indices to the
+	'''
+	A TreeView wrapper that initializes its columns, renderers and properties through
+	the use of a configuration structure. This ConfigTreeView is a little different
+	from the original design. The TreeView itself handles assigning indices to the
 	data model, which is determined by the configuration structure sent in this
-	`gtk.TreeView`'s init method.
-	The API for this configuration data structure is located in 'docs/config_api_notes.txt'
-	"""
-	
+	TreeView's init method.
+	'''	
+	minimum_keys = {'index_names'} #The minimum top-level keys that are required in the config structure
+	accepted_keys = {"treeview", "treemodel", "column_order", "macros", "columns"} | minimum_keys#The accepted top-level keys that can be in a config structure	
 	def __init__(self, config ):
 		'''
-		
 		This treeview is built from a configuration data structure. The configuration
 		gives enough information to build all of the TreeView's columns,
 		creates headers, create and pack cellrenderers, apply attributes
@@ -50,12 +51,10 @@ class ConfigTreeView( gtk.TreeView ):
 		of the indices that the columns require to give to the data model.
 		
 		:param `config`:
-			The configuration structure( a python dict) that will build the underlying 
+			The configuration structure( a python dict) that will build the underlying
 			gtk.TreeView.Check out :doc:`HOW-TO create a config <how_to_config_file>`
 			for info on how to build a configuration structure
-
 		'''
-		
 		gtk.TreeView.__init__(self)
 		
 		#Open json dict
@@ -66,11 +65,11 @@ class ConfigTreeView( gtk.TreeView ):
 				fp.seek(0)
 				self.config = simplejson.loads( fp.read() )
 			else:
-				raise InvalidFilePathError( "'%s' is not a valid filepath"%config )
+				raise InvalidFilePathError( "'%s' is not a valid filepath" % config )
 			
 		elif isinstance( config, dict ):
 			#It's a config dict already, so set self.config
-			self.config = config
+			self.config = dict(config)
 
 		self.macros = None #The macros structure that will be used to set properties
 		self.index_map = {}#Maps column names/index variables to an index in a TreeModel row
@@ -79,18 +78,46 @@ class ConfigTreeView( gtk.TreeView ):
 		self.args = []#Positional args for a custom ConfigTreeView to handle
 		self.kwargs = {}#Keyword args for a custom ConfigTreeView to handle
 		self.selection_modes = ('SELECTION_SINGLE', 'SELECTION_NONE', 'SELECTION_BROWSE', 'SELECTION_MULTIPLE')#Acceptable selection modes for a GtkTreeView
+		#self._apply_config()
 
 	def _apply_config(self):
 		'''
 		Validate the configuration structure, build the treeview's columns, headers,
 		renderers and properties based on the configuration data structure
 		'''
+		#Validate the config structure
+		#1 - Check that the minimum keys to create the treeview are present
+		for key in self.minimum_keys:
+			if key not in self.config:
+				raise ConfigValidationError("Validation Error!: %s is missing from config!!!!"%key)
+		#2 - Check that the keys that are present in config are all valid
+		for key in self.config:
+			if key not in self.accepted_keys:
+				raise ConfigValidationError("Validation Error!: %s is not a valid key in config!!!!" % key)
+		#3 - Validate index_names
+		#	This includes checking for columns that aren't defined in index_names, but are present in "columns"
+		#	Also the reverse, a warning is issued if the column is never used
+		#		NOTE: this should be intelligent and skip over values that are defined
+
 		#Build the index map--this structure will be a mapping of cellrenderer
 		#property names to the index in a TreeModel's row that the property will
 		#be stored in.
-		self.index_map = self.config.get('index_names', None )
+		self.index_map = copy.deepcopy(self.config.get('index_names', {})) #Need to make deepcopy because we don't want any reference to the original structure.
+		
+		#Validate the index_map
+		for column in self.config.get('columns', {}):
+			if column not in self.index_map:
+				self.index_map = {}#Reset index_map
+				raise ConfigValidationError("Validation Error!: '%s' is defined as a column in the config structure, but it isn't defined in `index_names`!" % column)
+		
+		for column, value in self.index_map.iteritems():
+			if not isinstance(value, basestring) and (isinstance(value, list) or isinstance(value, dict)):
+				if column not in self.config.get('columns', {}):
+					#Just print a warning that a defined column is never used
+					print "Warning!: '%s' is defined as a column in `index_names`, but is never used" % column
+
 		index_names = self.index_map
-		if index_names is None:
+		if index_names is {}:
 			raise KeyError( "`index_names` doesn't exist in the config" )
 		for key,value in index_names.iteritems():
 			if isinstance( value, dict ):
@@ -104,7 +131,15 @@ class ConfigTreeView( gtk.TreeView ):
 				value = self._get_type_index(value)
 				self.index_vars.append( key )#Update the index_variables list
 				self.index_map[key] = value
-		
+
+		"""
+		#Debugging
+		print "Okay, successfully created index map, here it is!:"
+		pprint.pprint( self.index_map )
+		print "*"*50
+		print 'Types:'
+		pprint.pprint(self.types)
+		"""
 		#Handle args--for custom ConfigTreeViews
 		#Check for $index args or kwargs...validate them
 		#It's valid if the 'dotted' key evaluates to an actual value
@@ -137,7 +172,6 @@ class ConfigTreeView( gtk.TreeView ):
 		#--This includes its properties
 		#--But also a column's renderers, and its properties, and indices
 		self._init_treeview_columns()
-
 		
 	def _init_treeview_columns(self):
 		'''
@@ -247,15 +281,15 @@ class ConfigTreeView( gtk.TreeView ):
 				for idx, renderer in enumerate(renderers):
 					key_list = [col_name]
 					key_list.append( idx )
-					self._setup_renderer(col, renderer, key_list)
+					self.__setup_renderer(col, renderer, key_list)
 					
 			elif isinstance( renderers, dict ):
 				key_list = [col_name]
-				self._setup_renderer( col, renderers, key_list )
+				self.__setup_renderer( col, renderers, key_list )
 			else:
 				raise ValueError( "ValueError!:  `renderers` must be either of type list or dict, not %s"%type(renderers))
 
-	def _setup_renderer( self, col, renderer, key_list ):
+	def __setup_renderer( self, col, renderer, key_list ):
 		'''
 		Initalize the GtkTreeViewColumn's renderers. This includes setting their properties, adding
 		data model attributes and, if necessary, creating them from a custom class, with their
@@ -304,8 +338,9 @@ class ConfigTreeView( gtk.TreeView ):
 					
 			#Get args and kwargs
 			#If they're None, then do not pass anything to the constructor
-			args = renderer.get( 'args', None )
-			kwargs = renderer.get('kwargs', None )
+			args = renderer.get('args', None)
+			kwargs = renderer.get('kwargs', None)
+			
 			#Create CellRenderer
 			cell = None
 			if args is None and kwargs is None:
@@ -324,7 +359,7 @@ class ConfigTreeView( gtk.TreeView ):
 			pack = renderer.get('pack', 'pack_start' )
 			expand = renderer.get( 'expand', True )
 			pack = getattr( col, pack )
-			pack( cell, expand )
+			pack(cell, expand)
 
 			#Setup indices
 			indices = renderer.get( 'indices', None )
@@ -343,10 +378,8 @@ class ConfigTreeView( gtk.TreeView ):
 		'''
 		Attempts to get the index for the given attribute at the path defined
 		in key_list
-		
-		`key_list[list]`
-			A list of the nested indices to access in the index_map
-
+		Args:
+			key_list[list]: A list of the nested indices to access in the index_map
 		'''
 		index_map = self.index_map
 		keys = ""
@@ -374,7 +407,7 @@ class ConfigTreeView( gtk.TreeView ):
 				try:
 					widget.set_property( prop, value )
 				except TypeError:
-					print "Warning: [%s] is not a valid property for `%s`, skipping it."%(prop, widget)
+					print "Warning: [%s] is not a valid property for `%s`, skipping it." % (prop, widget)
 
 	def _init_treeview(self):
 		'''
@@ -439,8 +472,7 @@ class ConfigTreeView( gtk.TreeView ):
 		widget "*bg_color*" style "bg_color"
 		""" % color)
 		self.set_rules_hint( False )
-		self.set_name( 'bg_color' )			
-		
+		self.set_name( 'bg_color' )
 
 	def _handle_args( self, *args, **kwargs ):
 		'''
@@ -456,9 +488,8 @@ class ConfigTreeView( gtk.TreeView ):
 			Keyword arguments passed in typical python fashion.
 
 		'''
-		#print "Args: ",args
-		#print "Kwargs: ",kwargs		
-		pass
+		print "Args: ",args
+		print "Kwargs: ",kwargs		
 
 	def _evaluate_dotted_key( self, value ):
 		'''
@@ -493,6 +524,7 @@ class ConfigTreeView( gtk.TreeView ):
 		'''
 		if not isinstance( a_dict, dict ):
 			raise TypeError( "TypeError: Multiple CellRenderers can only be represented by a list of dicts in the config file, not a list of '%s'"% a_dict )
+		
 		for key, value in a_dict.iteritems():
 			#print value
 			val = self._get_type_index( value )
@@ -535,7 +567,7 @@ class ConfigTreeView( gtk.TreeView ):
 			value = attr
 
 		else:
-			raise ValueError( "ValueError!: value at config['index_map'][%s] is not a valid value. Looking for a dict, list, string or a valid type"%value )
+			raise ValueError( "ValueError!: value at config['index_names'][%s] is not a valid value. Looking for a dict, list, string or a valid type"%value )
 
 		#Update the new type
 		self.types.append( value )
@@ -550,6 +582,50 @@ class ConfigTreeView( gtk.TreeView ):
 				if self.macros.get(prop, None ) is not None:
 					self._set_properties( widget, self.macros[prop] )
 				else:
-					print "Warning: [%s] is not a valid macro, ignoring it." % prop	
+					print "Warning: [%s] is not a valid macro, ignoring it." % prop
+
+	def get_treemodel(self):
+		'''
+		Initializes a treemodel that's special for this ConfigTreeView as defined
+		in the `treemodel` key of the config structure.
+		If there is no `treemodel` defined in the config structure, then a ``GtkListStore``
+		is initialized with the types list created from initializing the ConfigTreeView.
+		If a custom ``GtkTreeModel`` is defined in the config structure, then it
+		must have a function ``_handle_args()`` that should assign `args` and `kwargs`,
+		and also any other post initialization stuff.
+		
+		:return:
+			an instance of `GtkTreeModel` that is initialized with
+			the correct `types` array that is generated from the config structure.
+			If a custom treemodel was supplied in the config structure, then this
+			function attempts to create the custom `GtkTreeModel` and initialize
+			it with any arguments supplied in the config structure. If one doesn't
+			exist, then a `GtkListStore` is initialized and returned.
+		
+		'''
+		treemodel_dict = self.config.get('treemodel', {})
+		if treemodel_dict.get('class', None) is not None:
+			try:
+				treemodel = getattr(sys.modules[__name__], treemodel_dict['class'])
+			except Exception as e:
+				print 'Warning there was an error in get_treemodel:\n%s\nFalling back to GtkListStore' % e
+				treemodel = None
+			
+			if treemodel is None:
+				treemodel = gtk.ListStore(*self.types)
+			else:
+				treemodel = treemodel(*self.types)
+				try:
+					treemodel._handle_args(*treemodel_dict.get('args', []), **treemodel_dict.get('kwargs', {}))
+				except Exception as e:
+					print 'Warning! Thre was an error trying to call _handle_args on the treemodel:\n%s' % e
+		else:
+			treemodel = gtk.ListStore(*self.types)
+		
+		return treemodel
+			
+			
+		
+
 
 
